#include <linux/module.h>

#include <rtai.h>
#include <rtai_nam2num.h>
#include <rtai_sched.h>

#define BASEPORT 0x378
#define INTERRUPT 7

static int end;

static int startzeit;
static int endzeit;
static int zeitdiff;

// Interrupt-Handler, wird bei ausgelstem Interrupt aufgerufen
static void irq_handler(void) {
  endzeit = rt_get_cpu_time_ns();
  zeitdiff  = endzeit  - startzeit;
  rt_printk("Interrupt empfangen. Latenz = %d ns\n", zeitdiff);
  rt_ack_irq(INTERRUPT);
}


// Erzeugt Interrupt und speichert Start-Zeit.
// Dazu muss ein beliebiges Datenpin (Pin 2-9)
// des ParPort mit dem ACK-Pin (Pin 10) verbunden sein.
static void generate_irq(void) {
  rt_printk("Generiere Interrupt ...\n");
  outb_p(0, BASEPORT);
  startzeit = rt_get_cpu_time_ns();
  outb_p(255, BASEPORT);
}



// INIT
int xinit_module(void) {

  int ret;
  end=0;

  rt_printk("ParPort Interrupt Latenz-Test gestartet...\n");
  ret = rt_request_global_irq(INTERRUPT, (void *)irq_handler);
  rt_enable_irq(INTERRUPT);
  outb_p(0x10, BASEPORT + 2); // IRQ im ParPort aktivieren

  generate_irq();
  return 0;
}



// EXIT
void xcleanup_module(void) {

  end=1;
  rt_disable_irq(INTERRUPT);
  rt_free_global_irq(INTERRUPT);
  rt_printk("ParPort Interrupt Latenz-Test beendet...\n");
}


///////////////////////////////

module_init(xinit_module);
module_exit(xcleanup_module);

MODULE_LICENSE("GPL");

