// ===> versuch3/rtai-userspace.c

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <errno.h>
#include <sys/mman.h>
#include <sys/io.h>

#include <rtai_lxrt.h>
#include <rtai_sem.h>
#include <rtai_usi.h>

#define INTERRUPT 7
#define BASEPORT  0x378

static RT_TASK *maint, *handler;
static SEM *semaphore;
static int thread;
static volatile int maxcnt = 5;
static volatile int end = 1;
static volatile int ovr, intcnt;

static void *irq_handler(void *args) {
	if (!(handler = rt_task_init_schmod(nam2num("HAND"),0,0,0,SCHED_FIFO,0xF))) {
		printf("CANNOT INIT HANDLER TASK\n"); exit(1); }
	rt_allow_nonroot_hrt();
	mlockall(MCL_CURRENT | MCL_FUTURE);
	rt_make_hard_real_time();
	end=0;
	// IRQ aktivieren
	rt_request_irq_task(INTERRUPT, handler, RT_IRQ_TASK, 1);
	rt_startup_irq(INTERRUPT);
	rt_enable_irq(INTERRUPT);

	while ( !end && (ovr != RT_IRQ_TASK_ERR) ) {
		do {
			// Warte auf Interrupt:
			ovr = rt_irq_wait(INTERRUPT);
			if (ovr == RT_IRQ_TASK_ERR) break;
			if (end) break;
			if (ovr > 0) { rt_sem_signal(semaphore); }
			intcnt++;
			rt_sem_signal(semaphore); // main() benachrichtigen
			rt_ack_irq(INTERRUPT);
		} while (ovr > 0);
		rt_pend_linux_irq(INTERRUPT);
	}
	rt_release_irq_task(INTERRUPT);
	rt_make_soft_real_time();
	rt_task_delete(handler);
	intcnt = maxcnt;
	return 0;
}

int main(void) {
	// Haupt-Task erstellen
	if (!(maint = rt_task_init(nam2num("MAIN"), 1, 0, 0))) { 
		printf("CANNOT INIT MAIN TASK\n"); exit(1);	}
	// Semaphore zum Benachrichtigen von main() wenn Interrupt auftritt erstellen
	if (!( semaphore  = rt_sem_init(nam2num("SEM"), 0))) {
		printf("CANNOT INIT SEMAPHORE\n"); exit(1); }
	// Berechtigung f. Zugriff auf ParPort aus Userspace holen
	if (iopl(3)) {
		printf("iopl err ? keine rechte f. parport bekommen\n");
		rt_task_delete(maint); rt_sem_delete(semaphore); exit(1); }
	// ParPort in Interrupt-Modus versetzen & auf Ausgabe einstellen
	outb_p(0x10, BASEPORT + 2);
	// Thread erstellen
	thread = rt_thread_create(irq_handler, NULL, 10000);
	// warten bis Thread im harten Echtzeitmodus ist
	while (end) { usleep(100000); }
	// Bei Benachrichtigung Anzahl IRQ ausgeben - solange max. Anzahl nicht erreicht 
	while (intcnt < maxcnt) {
		rt_sem_wait(semaphore); // WARTE auf Semaphore
		printf("Bisherige Anzahl IRQ: %d / Overrun: %d\n", intcnt, ovr);
	}
	end=1;
	// Finalen Interrupt erzeugen um rt_irq_wait zu befreien
	outb_p(0, BASEPORT); outb_p(255, BASEPORT); outb_p(0, BASEPORT);
	rt_release_irq_task(INTERRUPT);
	rt_thread_join(thread);
	rt_task_delete(maint);
	rt_sem_delete(semaphore);
	return 0;
}

